// one line to give the program's name and an idea of what it does.
// Copyright (C) 2005  name of Borja Snchez Zamorano
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

using System;
using System.Collections;
using System.Drawing;
using Forms = System.Windows.Forms;

namespace MonoCalendar.ListView
{
	public delegate void ItemHandler(ListViewItem item);

	public class ListView : System.Windows.Forms.Control {
		private MonoCalendar.ListView.Content Content1;
		private Forms.VScrollBar vScrollBar1;
		private MonoCalendar.ListView.Title Title1;

		public ListView()
		{
			this.SuspendLayout();
			Content1 = new Content();
			Title1 = new Title();
			vScrollBar1 = new Forms.VScrollBar();
			this.Controls.Add(this.vScrollBar1);
			this.Controls.Add(this.Content1);
			this.Controls.Add(this.Title1);
			this.ResumeLayout(false);

			Content1.OnCheckedChanged += new ItemHandler(CheckedChanged);
			Content1.OnAfterLabelEdit += new ItemHandler(AfterLabelEdit);
			Content1.OnSelectedChanged += new System.EventHandler(SelectedChanged);
			Content1.KeyDown += new System.Windows.Forms.KeyEventHandler(this.CCKeyDown);
			this.Enter += new System.EventHandler(this.ListViewEnter);
			this.Leave += new System.EventHandler(this.ListViewLeave);
			this.Resize += new System.EventHandler(this.ListViewResize);
			this.vScrollBar1.ValueChanged += new System.EventHandler(this.VScrollBar1ValueChanged);
		}

		void ListViewLeave(object sender, System.EventArgs e)
		{
			Content1.IsFocused = false;
		}
		
		void ListViewEnter(object sender, System.EventArgs e)
		{
			Content1.IsFocused = true;
		}

		void CCKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			OnKeyDown(e);
		}

		void Resize2()
		{
			if ((this.ClientRectangle.Height<1)||(this.ClientRectangle.Width<1)) return;
 			vScrollBar1.SuspendLayout();
			Content1.SuspendLayout();
			Title1.SuspendLayout();
			
			Title1.SetBounds(0,0,this.ClientRectangle.Width,16);
			vScrollBar1.Height = this.ClientRectangle.Height - Title1.Height;
			if (vScrollBar1.Height<1) vScrollBar1.Height=1;
			if ((this.ClientRectangle.Height- Title1.Height) < Content1.HeightList) {
				vScrollBar1.Visible = true;
				vScrollBar1.Left = this.ClientRectangle.Width - vScrollBar1.Width;
				vScrollBar1.Top = Title1.Height;
				Content1.SetBounds(0, Title1.Height, this.ClientRectangle.Width - vScrollBar1.Width, vScrollBar1.Height);
			} else {
				vScrollBar1.Visible = false;
				Content1.SetBounds(0, Title1.Height, this.ClientRectangle.Width, vScrollBar1.Height);
			}

			int max = Content1.Items.Count - (Content1.Height/Content1.HeightItem);
			if (max < 0) max = 0;
			vScrollBar1.LargeChange = 1;
			vScrollBar1.Maximum = max;
			
			Title1.ResumeLayout(true);
			Content1.ResumeLayout(true);
			vScrollBar1.ResumeLayout(true);
		}

		void ListViewResize(object sender, System.EventArgs e)
		{
			Resize2();
		}

		void VScrollBar1ValueChanged(object sender, System.EventArgs e)
		{
			Content1.ScrollValue = vScrollBar1.Value;
		}

		public void UpdateList()
		{
			Content1.UpdateList();
			Resize2();
		}
		
		public void EditLabel()
		{
			 Content1.EditLabel();
		}

		public ListViewItem Selected {
			get {
				return Content1.Selected;
			}
			set {
				if (Content1.Selected != value) {
					Content1.Selected = value;
					UpdateList();
					SelectedChanged(this, null);
				}
			}
		}

		public ArrayList Items {
			get {
				return Content1.Items;
			}
			set {
				Content1.Items = value;
			}
		}

		private void CheckedChanged(ListViewItem item)
		{
			if (OnCheckedChanged != null) OnCheckedChanged(item);
		}
		
		private void AfterLabelEdit(ListViewItem item)
		{
			if (OnAfterLabelEdit != null) OnAfterLabelEdit(item);
		}

		private void SelectedChanged(object o, System.EventArgs e)
		{
			if (OnSelectedChanged != null) OnSelectedChanged(this, e);
		}

		public event ItemHandler OnCheckedChanged;
		public event ItemHandler OnAfterLabelEdit;
		public event EventHandler OnSelectedChanged;
		
		public string Title {
			get {
				return Title1.NameTitle;
			}
			set {
				Title1.NameTitle = value;
			}
		}
		
	}
}
